@@ -1,27 +1,27 @@
 /* -*- mode: C; mode: fold; -*- */
-/* This file is a MESS.  For that reason, I wrote sltcp.c which I regard as 
+/* This file is a MESS.  For that reason, I wrote sltcp.c which I regard as
  * much cleaner.  I tried to clean this up but with at least 4 different
  * TCP/IP implementations for VMS as well as TLI and BSD, there is not too
- * much hope.  The routines in sltcp.c should be adequate for most Unix 
+ * much hope.  The routines in sltcp.c should be adequate for most Unix
  * systems as well as OS/2.  Until VMS standardizes its TCP/IP interface
  * the routines in this file will have to be used.
- * 
- * Much of this file has been totally re-written.  Now all read/writes are 
- * to a file descriptor.  All reads take place via the function 
+ *
+ * Much of this file has been totally re-written.  Now all read/writes are
+ * to a file descriptor.  All reads take place via the function
  * 'client_read' and all writes are done in 'client_write'.  These functions
  * call the appropriate low level functions to perform the actual operation.
  * Specifically, client_read calls:
- *  
+ *
  *     socket_read (MULTINET)
- *     do_netlib_read  (NETLIB)   [Note: this function needs to be created.]
+ *     do_netlib_read  (NETLIB)
  *     read   (everything else)
- * 
+ *
  * and client_write calls:
- * 
+ *
  *     netlib_write (NETLIB)
  *     socket_write (MULTINET)
  *     write  (everything else)
- * 
+ *
  * JED.
  */
 
@@ -40,9 +40,8 @@
  *
  */
 
-
 #if 0
-# define DEBUG_FILE "slrn.log" 
+# define DEBUG_FILE "slrn.log"
 #endif
 
 #include "config.h"
@@ -91,7 +90,6 @@
 
 #include <ctype.h>
 
-
 #ifdef TLI
 /*{{{ TLI includes */
 # include	<fcntl.h>
@@ -139,7 +137,6 @@
 #  endif /* !EXCELAN */
 # endif /* !VMS */
 #endif /* !TLI */
- 
 
 #ifdef HAVE_SYS_SOCKET_H
 # ifndef SLRN_LOADED_SYS_SOCKET_H
@@ -198,14 +195,12 @@ int socket( int, struct sockproto *, struct sockaddr_in *, int );
 # define NNTP_PORT 119
 #endif
 
-
 #ifdef NEEDS_EXTERN_DECLARATIONS
 # define DECLARE_EXTERN(x) extern x;
-#else 
+#else
 # define DECLARE_EXTERN(x)
 #endif
 
-
 #ifdef NETLIB
 static void *Socket_Fd;
 #else
@@ -218,7 +213,6 @@ static FILE *Debug_Fp;
 
 static int Server_Has_Been_Closed = 1;
 
-
 static void client_close_server(void);
 
 static int get_tcp_socket(char *, int);
@@ -241,10 +235,10 @@ static int client_server_init_1 (char *machine, int port)
    int	sockt_rd;
 #ifdef DECNET
    char	*cp;
-   
+
    cp = slrn_strchr(machine, ':');
-   
-   if (cp && cp[1] == ':') 
+
+   if (cp && cp[1] == ':')
      {
 	*cp = '\0';
 	sockt_rd = get_dnet_socket(machine);
@@ -254,23 +248,22 @@ static int client_server_init_1 (char *machine, int port)
 #else
    sockt_rd = get_tcp_socket (machine, port);
 #endif
-   
+
    if (sockt_rd < 0)
      return (-1);
 
 #ifndef NETLIB
    Socket_Fd = sockt_rd;
 #endif
-   
+
    /* Now get the server's signon message */
 #ifdef DEBUG_FILE
    Debug_Fp = fopen (DEBUG_FILE, "w");
 #endif
-   
+
    return 0;
 }
 
-
 /*}}}*/
 
 /*{{{ get_tcp_socket */
@@ -293,7 +286,7 @@ static int get_tcp_socket(char *machine, int nntpport)
    int	s = -1;
    struct	sockaddr_in s_in;
 #endif
-#ifdef TLI 
+#ifdef TLI
    char 	*t_alloc();
    struct	t_call	*callptr;
    /*
@@ -315,7 +308,7 @@ static int get_tcp_socket(char *machine, int nntpport)
 
    if (nntpport < 0) nntpport = NNTP_PORT;
    s_in.sin_port = htons((unsigned short) nntpport);
-   
+
    if (!isdigit(*machine) ||
        (long)(s_in.sin_addr.s_addr = inet_addr(machine)) == -1)
      {
@@ -339,43 +332,43 @@ static int get_tcp_socket(char *machine, int nntpport)
 	t_close(s);
 	return(-1);
      }
-   
+
    callptr->addr.maxlen = sizeof(s_in);
    callptr->addr.len = sizeof(s_in);
    callptr->addr.buf = (char *) &s_in;
    callptr->opt.len = 0;			/* no options */
    callptr->udata.len = 0;			/* no user data with connect */
-   
+
    /*
     * Connect to the server.
     */
-   if (t_connect(s, callptr, (struct t_call *) 0) < 0) 
+   if (t_connect(s, callptr, (struct t_call *) 0) < 0)
      {
 	t_error("t_connect");
 	t_close(s);
 	return(-1);
      }
-   
+
    /*
     * Now replace the timod module with the tirdwr module so that
     * standard read() and write() system calls can be used on the
     * descriptor.
     */
-   
+
    if (ioctl(s,  I_POP,  (char *) 0) < 0)
      {
 	perror("I_POP(timod)");
 	t_close(s);
 	return(-1);
      }
-   
+
    if (ioctl(s,  I_PUSH, "tirdwr") < 0)
      {
 	perror("I_PUSH(tirdwr)");
 	t_close(s);
 	return(-1);
      }
-   
+
 #else /* !TLI */
 # ifdef NETLIB
    unsigned short port, nntp_port;
@@ -383,11 +376,11 @@ static int get_tcp_socket(char *machine, int nntpport)
    struct INADDRDEF addr;
    struct SINDEF s_in;
    unsigned int rc, size;
-   
+
    if (nntpport < 0) nntpport = NNTP_PORT;
    nntp_port = (unsigned short) nntpport;
    port = netlib_hton_word(&nntp_port);
-   
+
    rc = netlib_socket(&Socket_Fd);
    if (!$VMS_STATUS_SUCCESS(rc))
      return(-1);
@@ -440,16 +433,16 @@ static int get_tcp_socket(char *machine, int nntpport)
    DECLARE_EXTERN(unsigned long inet_addr())
 #   ifndef UCX
    struct servent sp_buffer;
-   
+
    if (nntpport >= 0)
      {
 	sp = &sp_buffer;
 	memset ((char *) sp, 0, sizeof (struct servent));
 	sp->s_port = htons((unsigned short)nntpport);
      }
-   else 
+   else
      {
-	if ((sp = getservbyname("nntp", "tcp")) ==  NULL) 
+	if ((sp = getservbyname("nntp", "tcp")) ==  NULL)
 	  {
 	     fprintf(stderr, "nntp/tcp: Unknown service.\n");
 	     return (-1);
@@ -464,7 +457,7 @@ static int get_tcp_socket(char *machine, int nntpport)
    if (!isdigit(*machine) ||
        (long)(defaddr.s_addr = inet_addr(machine)) == -1)
      hp = gethostbyname(machine);
-   else 
+   else
      {
 	/* Raw ip address, fake  */
 	(void) strcpy(namebuf, machine);
@@ -478,12 +471,12 @@ static int get_tcp_socket(char *machine, int nntpport)
 	def.h_aliases = 0;
 	hp = &def;
      }
-   if (hp == NULL) 
+   if (hp == NULL)
      {
 	fprintf(stderr, "%s: Unknown host.\n", machine);
 	return (-1);
      }
-   
+
    memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = hp->h_addrtype;
    s_in.sin_port = sp->s_port;
@@ -491,7 +484,7 @@ static int get_tcp_socket(char *machine, int nntpport)
    memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = AF_INET;
 #  endif /* EXCELAN */
-   
+
    /*
     * The following is kinda gross.  The name server under 4.3
     * returns a list of addresses, each of which should be tried
@@ -501,21 +494,21 @@ static int get_tcp_socket(char *machine, int nntpport)
     * We use this to figure out whether to include the NS specific
     * code...
     */
-   
+
 #  ifdef	h_addr
-   
+
    /* get a socket and initiate connection -- use multiple addresses */
-   
-   for (cp = hp->h_addr_list; cp && *cp; cp++) 
+
+   for (cp = hp->h_addr_list; cp && *cp; cp++)
      {
 	s = socket(hp->h_addrtype, SOCK_STREAM, 0);
-	if (s < 0) 
+	if (s < 0)
 	  {
 	     perror("socket");
 	     return (-1);
 	  }
 	memcpy ((char *)&s_in.sin_addr, *cp, hp->h_length);
-	
+
 	if (x < 0)
 	  fprintf(stderr, "trying %s\n",
 		  (char *) inet_ntoa(s_in.sin_addr));
@@ -527,7 +520,7 @@ static int get_tcp_socket(char *machine, int nntpport)
 	perror("");
 	(void) close(s);
      }
-   if (x < 0) 
+   if (x < 0)
      {
 	fprintf(stderr, "giving up...\n");
 	return (-1);
@@ -545,42 +538,42 @@ static int get_tcp_socket(char *machine, int nntpport)
    if (nntpport < 0) nntpport = NNTP_PORT;
    s_in.sin_port = htons((unsigned short) nntpport);
    /* set up addr for the connect */
-   
-   if ((s_in.sin_addr.s_addr = rhost(&machine)) == -1) 
+
+   if ((s_in.sin_addr.s_addr = rhost(&machine)) == -1)
      {
 	fprintf(stderr, "%s: Unknown host.\n", machine);
 	return (-1);
      }
    /* And then connect */
-   
-   if (connect(s, (struct sockaddr *)&s_in) < 0) 
+
+   if (connect(s, (struct sockaddr *)&s_in) < 0)
      {
 	perror("connect");
 	(void) close(s);
 	return (-1);
      }
 #   else /* not EXCELAN */
-   if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
+   if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      {
 	perror("socket");
 	return (-1);
      }
-   
+
    /* And then connect */
-   
+
    memcpy ((char *) &s_in.sin_addr, hp->h_addr, hp->h_length);
-   if (connect(s, (struct sockaddr *) &s_in, sizeof(s_in)) < 0) 
+   if (connect(s, (struct sockaddr *) &s_in, sizeof(s_in)) < 0)
      {
 	perror("connect");
 	(void) close(s);
 	return (-1);
      }
-   
+
 #   endif /* !EXCELAN */
 #  endif /* !h_addr */
 # endif /* !NETLIB */
 #endif /* !TLI */
-   
+
 #ifndef NETLIB
    return (s);
 #endif
@@ -608,10 +601,10 @@ int get_dnet_socket (char *machine)
    int	s, area, node;
    struct	sockaddr_dn sdn;
    struct	nodeent *getnodebyname(), *np;
-   
+
    memset ((char *) &sdn, 0, sizeof(sdn));
-   
-   switch (s = sscanf( machine, "%d%*[.]%d", &area, &node )) 
+
+   switch (s = sscanf( machine, "%d%*[.]%d", &area, &node ))
      {
       case 1:
 	node = area;
@@ -624,13 +617,13 @@ int get_dnet_socket (char *machine)
 	sdn.sdn_add.a_addr[1] = node / 256;
 	break;
       default:
-	if ((np = getnodebyname(machine)) == NULL) 
+	if ((np = getnodebyname(machine)) == NULL)
 	  {
 	     fprintf(stderr,
 		     "%s: Unknown host.\n", machine);
 	     return (-1);
 	  }
-	else 
+	else
 	  {
 	     memcpy((char *) sdn.sdn_add.a_addr, np->n_addr,
 		   np->n_length);
@@ -643,22 +636,22 @@ int get_dnet_socket (char *machine)
    sdn.sdn_flags = 0;
    sdn.sdn_objnamel = strlen("NNTP");
    memcpy (&sdn.sdn_objname[0], "NNTP", sdn.sdn_objnamel);
-   
-   if ((s = socket(AF_DECnet, SOCK_STREAM, 0)) < 0) 
+
+   if ((s = socket(AF_DECnet, SOCK_STREAM, 0)) < 0)
      {
 	nerror("socket");
 	return (-1);
      }
-   
+
    /* And then connect */
-   
-   if (connect(s, (struct sockaddr *) &sdn, sizeof(sdn)) < 0) 
+
+   if (connect(s, (struct sockaddr *) &sdn, sizeof(sdn)) < 0)
      {
 	nerror("connect");
 	close(s);
 	return (-1);
      }
-   
+
    return (s);
 }
 /*}}}*/
@@ -666,6 +659,52 @@ int get_dnet_socket (char *machine)
 
 /*{{{ Low-Level Read Routines */
 
+#ifdef NETLIB
+/*
+ * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ *  do_netlib_read
+ *
+ *  Purpose:
+ *	 Read data from the network via NETLIB
+ *
+ *  Inputs:
+ *	 s	NETLIB channel
+ *	 len	length of output buffer
+ *
+ *  Outputs:
+ *	 buf	Pointer to output buffer
+ *
+ *  Return Value:
+ *	 Number of bytes read, or
+ *	 -1	Error
+ *
+ *  Comments:
+ *
+ *  Author:
+ *	 1.0	Andy Harper, Kings College London	Jan 1998
+ * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ */
+static int do_netlib_read(void *s, unsigned char *buf, unsigned int len)
+{
+   struct dsc$descriptor bufdesc = {0, DSC$K_DTYPE_T, DSC$K_CLASS_S, NULL};
+   struct { unsigned short status, length; unsigned dummy; } iosb;
+   int status;
+
+   /* Assign values to the descriptor to specify buffer */
+   bufdesc.dsc$w_length = len;
+   bufdesc.dsc$a_pointer= (char *)buf;
+   status = netlib_read(s, &bufdesc, NULL, NULL, 0, 0, &iosb, NULL, 0);
+
+# if NETLIBDEBUG
+			/* For debugging, print status and length */
+   printf("netlib status = 0x%08x, iosb status = 0x%08x, return length = 0x%08x\n", status, iosb.status, iosb.length);
+# endif
+
+			/* Return length, or -1 on any non normal status */
+   return ((status == SS$_NORMAL) && (iosb.status == SS$_NORMAL)) ? iosb.length : -1;
+}
+#endif				       /* NETLIB */
+
 #define TIMEOUT 60
 #define BUFSIZE 8192
 
@@ -684,22 +723,21 @@ static void cancel_read (int sig_unused)
    slrn_error ("CANCEL_READ: NNTP read failed.");
 }
 
-
 static int client_read (void)
 {
    int nb;
 #if TIMEOUT
    void (*old_sigalrm)(int);
 #endif
-   
+
    reset_read_buffer ();
 #if TIMEOUT
    old_sigalrm = signal (SIGALRM, cancel_read);
    alarm (TIMEOUT);
 #endif
-   
+
 #ifdef NETLIB
-   nb = do_netlib_read (Socket_Fd, Read_Buffer, BUFSIZE);
+   nb = do_netlib_read (&Socket_Fd, Read_Buffer, BUFSIZE);
 #else
 # ifdef MULTINET
    nb = socket_read (Socket_Fd, Read_Buffer, BUFSIZE);
@@ -710,55 +748,54 @@ static int client_read (void)
 	if (errno == EINTR) continue;
 #  endif
 	break;
-     }   
+     }
 # endif				       /* MULTINET */
 #endif				       /* NETLIB */
-   
+
 #if TIMEOUT
    alarm(0);
    (void) signal (SIGALRM, old_sigalrm);
 #endif
-   
+
    if (nb <= 0)
      return -1;
-   
+
    Read_Buffer_Ptr_Max = Read_Buffer + nb;
-   
+
    return 0;
 }
 
-
 static int client_fgets (char *buf, int len)
 {
    unsigned char *r, *rmax;
    char *buf_max;
-   
+
    if (Server_Has_Been_Closed) return -1;
-   
+
    if (len == 0)
      {
 	*buf = 0;
 	return 0;
      }
-   
+
    buf_max = buf + (len - 1);	       /* allow room for \0 */
-   
+
    while (1)
      {
 	r = Read_Buffer_Ptr;
 	rmax = Read_Buffer_Ptr_Max;
-   
+
 	while (r < rmax)
 	  {
 	     unsigned char ch;
-	     
+
 	     if (buf == buf_max)
 	       {
 		  *buf = 0;
 		  Read_Buffer_Ptr = r;
 		  return 0;
 	       }
-	     
+
 	     ch = *r++;
 	     *buf++ = (char) ch;
 
@@ -770,7 +807,7 @@ static int client_fgets (char *buf, int len)
 	       }
 
 	  }
-	     
+
 	if (-1 == client_read ())
 	  return -1;
      }
@@ -778,13 +815,12 @@ static int client_fgets (char *buf, int len)
 
 /*}}}*/
 
-
 static void client_close_server(void) /*{{{*/
 {
 #if defined(VMS) && !defined(NETLIB)
    void (*save_sigalrm)(int);
 #endif
-   
+
    if (Server_Has_Been_Closed) return;
 
    if (
@@ -798,7 +834,7 @@ static void client_close_server(void) /*{{{*/
 	Server_Has_Been_Closed = 1;
 	return;
      }
-   
+
 #ifdef NETLIB
    netlib_close (&Socket_Fd);
    Socket_Fd = NULL;
@@ -822,15 +858,15 @@ static unsigned int client_write (char *buf, unsigned int len)
 {
    int n;
    unsigned int total;
-   
+
 #ifdef NETLIB
    struct dsc$descriptor dsc;
    struct NETLIBIOSBDEF iosb;
    int rc;
-   
+
    INIT_SDESC(dsc, len, buf);
-   netlib_write(&Socket_Fd, &dsc, 0,0, &iosb,0,0);
-   if ($VMS_STATUS_SUCCESS(rc))
+   rc = netlib_write(&Socket_Fd, &dsc, 0,0, &iosb,0,0);
+   if ($VMS_STATUS_SUCCESS(rc) && $VMS_STATUS_SUCCESS(iosb.iosb_w_status))
      total = iosb.iosb_w_count;
    else total = 0;
 #else
@@ -844,7 +880,7 @@ static unsigned int client_write (char *buf, unsigned int len)
 	if (n == -1)
 	  {
 #  ifdef EAGAIN
-	     if (errno == EAGAIN) 
+	     if (errno == EAGAIN)
 	       {
 		  sleep (1);
 		  continue;
@@ -868,7 +904,7 @@ static unsigned int client_write (char *buf, unsigned int len)
      }
 # endif				       /* MULTINET */
 #endif				       /* NETLIB */
-   
+
 #ifdef DEBUG_FILE
    if ((Debug_Fp != NULL) && (total != 0))
      {
@@ -876,18 +912,17 @@ static unsigned int client_write (char *buf, unsigned int len)
 	fflush (Debug_Fp);
      }
 #endif
-   
+
    return total;
 }
 
-
 static int client_fputs (char *s)
 {
    unsigned int len = strlen (s);
-   
+
    if (len != client_write (s, len))
      return -1;
-   
+
    return 0;
 }
 
@@ -900,25 +935,23 @@ static int client_vprintf (char *fmt, va_list ap)
 
 static int client_server_init (char *machine, int port)
 {
-   if (Server_Has_Been_Closed == 0) 
+   if (Server_Has_Been_Closed == 0)
      client_close_server ();
 
    reset_read_buffer ();
-   
+
    if (-1 == client_server_init_1 (machine, port))
      return -1;
-   
+
    Server_Has_Been_Closed = 0;
    return 0;
 }
 
-
 static int client_flush_output (void)
 {
    return 0;
 }
 
-
 static SLTCP_Type SLTCP_Buffer;
 
 int sltcp_fputs (SLTCP_Type *s, char *buf)
@@ -948,29 +981,27 @@ extern int sltcp_flush_output (SLTCP_Type *s)
 extern int sltcp_close (SLTCP_Type *s)
 {
    if (s == NULL) return -1;
-   
+
    (void) client_flush_output ();
    (void) client_close_server ();
 
    SLTCP_Buffer.tcp_fd = -1;
-   
+
    return 0;
 }
 
-
 extern unsigned int sltcp_write (SLTCP_Type *s, char *buf, unsigned int len)
 {
    if (s == NULL) return 0;
-   
+
    return client_write (buf, len);
 }
 
-
 extern SLTCP_Type *sltcp_open_connection (char *host, int port)
 {
    if (-1 == client_server_init (host, port))
      return NULL;
-   
+
    SLTCP_Buffer.tcp_fd = 1;
    return &SLTCP_Buffer;
 }
@@ -985,7 +1016,7 @@ extern int sltcp_close_socket (SLTCP_Type *s)
 {
    if (s == NULL) return -1;
    client_close_server ();
-   
+
    SLTCP_Buffer.tcp_fd = -1;
 
    return 0;
