@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -70,14 +69,15 @@ static char *Art_Help[] =
      "  TAB         Skip beyond quoted text.",
      "  ESC-r       Toggle ROT 13 decryption on and off.",
      "  g           Skip to next digest article.",
-     "  ESC Ctrl-C  Cancel article (you must be the author).",
+   "  ESC Ctrl-C  Cancel article (you must be the author).",
+   "  ESC Ctrl-S  Supersede article, allowing you to edit original text.",
 #if SLRN_HAS_GROUPLENS
      "  0           Assign article a GroupLens rating.",
 #endif
      "Posting, Replying, Printing, Piping, and Saving:",
      "  y           Send the article to the printer",
      "  P           Post a new article (NOT followup)",
-     "  ESC P       Post a postponed article",
+     "  ESC P       Post or edit a postponed article",
      "  f           Followup to current article.",
      "    Note: ESC 1 f ==> all headers will be inserted.",
      "          ESC 2 f ==> the message will be inserted without quoting.",
@@ -143,8 +143,9 @@ static char *Group_Help [] =
      "  3. Press the Capital-L key again to display only subscribed groups.",
      "Commands that affect the current group:",
      "  P                    Post an article to the current newsgroup",
-     "  ESC P                    Post a postponed article",
+     "  ESC P                Post a or edit postponed article",
      "  SPACE, RETURN        Select the current newsgroup.",
+     "   The following three variations permit viewing previously read articles:",
      "  ESC 1 SPACE          Select group with article number query.",
      "  ESC 2 SPACE          Select group but do not apply score.",
      "  ESC 3 SPACE          Select group with query but do not apply score.",
