@@ -1,24 +1,23 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -96,6 +95,7 @@ extern int h_errno;
 #include "server.h"
 #include "util.h"
 #include "ttymsg.h"
+#include "art.h"
 #include "chmap.h"
 
 #if SLRN_HAS_MIME
@@ -113,6 +113,7 @@ extern int h_errno;
 int Slrn_Full_Screen_Update = 1; 
 int Slrn_User_Wants_Confirmation = 1;
 int Slrn_Message_Present = 0;
+int Slrn_Abort_Unmodified = 0;
 
 #ifndef VMS
 char *Slrn_SendMail_Command;
@@ -274,7 +275,12 @@ static void redraw_message (void)
      
 static void vmessage_1 (int color, char *fmt, va_list ap)
 {
+#ifdef HAVE_VSNPRINTF
+   vsnprintf (Message_Buffer + 1, sizeof(Message_Buffer)-1, fmt, ap);
+#else
    vsprintf (Message_Buffer + 1, fmt, ap);
+#endif
+
    Message_Buffer[0] = (char) color;
    Slrn_Message_Present = 1;
    redraw_message ();
@@ -294,10 +300,8 @@ static void verror (char *fmt, va_list ap)
      {
 	slrn_tty_vmessage (stderr, fmt, ap);
      }
-   else
+   else if (Error_Present == 0)
      {
-	if (Error_Present) return;
-
 	slrn_clear_message ();
 	Error_Present = 1;
 	Beep_Pending = 1;
@@ -545,7 +549,7 @@ int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
    if (NULL != slrn_strchr (name, ':'))
      {
 	strcpy (dir, name);
-	return;
+	return 0;
      }
    home = getenv ("HOME");
    *dir = 0;
@@ -706,6 +710,10 @@ FILE *slrn_open_tmpfile (char *file, char *mode) /*{{{*/
 
 FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag) /*{{{*/
 {
+   char filebuf[SLRN_MAX_PATH_LEN];
+   
+   if (file == NULL) file = filebuf;
+
    slrn_make_home_filename (name, file);
    
 #ifdef VMS
@@ -724,6 +732,19 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag)
 
 /*}}}*/
 
+VFILE *slrn_open_home_vfile (char *name, char *file)
+{
+   char filebuf[SLRN_MAX_PATH_LEN];
+   
+   if (file == NULL) file = filebuf;
+
+   slrn_make_home_filename (name, file);
+   
+   return vopen (file, 4096, 0);
+}
+
+/*}}}*/
+
 int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char *subject) /*{{{*/
 {
    char buf [256 + 2 * SLRN_MAX_PATH_LEN];
@@ -733,7 +754,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
    
    if (edit && (Slrn_Batch == 0))
      {
-	if (slrn_edit_file (Slrn_Editor_Mail, file, editline) < 0) return -1;
+	if (slrn_edit_file (Slrn_Editor_Mail, file, editline, 1) < 0) return -1;
 	
 	while (1)
 	  {
@@ -743,7 +764,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	     rsp |= 0x20;
 	     if (rsp == 'n') return -1;
 	     if (rsp == 'y') break;
-	     if (slrn_edit_file (Slrn_Editor_Mail, file, 1) < 0) return -1;
+	     if (slrn_edit_file (Slrn_Editor_Mail, file, 1, 0) < 0) return -1;
 	  }
      }
    slrn_message_now ("Sending ...");
@@ -849,6 +870,83 @@ int _slrn_pclose (FILE *fp)
 }
 #endif				       /* SLRN_HAS_PIPING */
 
+#if SLRN_HAS_PIPING
+/* There appears to be no simply way of getting the command assocated with
+ * a pipe file descriptor.  Therefore, I will simply store them in a table.
+ */
+typedef struct _Pipe_Cmd_Table_Type
+{
+   char *cmd;
+   FILE *fp;
+   struct _Pipe_Cmd_Table_Type *next;
+}
+Pipe_Cmd_Table_Type;
+
+Pipe_Cmd_Table_Type *Pipe_Cmd_Table;
+   
+static int store_pipe_cmd (char *cmd, FILE *fp)
+{
+   Pipe_Cmd_Table_Type *p;
+
+   if (NULL == (cmd = SLang_create_slstring (cmd)))
+     return -1;
+   
+   p = (Pipe_Cmd_Table_Type *)slrn_malloc (sizeof (Pipe_Cmd_Table_Type), 1, 0);
+   if (p == NULL)
+     {
+	SLang_free_slstring (cmd);
+	return -1;
+     }
+   p->cmd = cmd;
+   p->fp = fp;
+   p->next = Pipe_Cmd_Table;
+   Pipe_Cmd_Table = p;
+   return 0;
+}
+
+static void delete_pipe_cmd (FILE *fp)
+{   
+   Pipe_Cmd_Table_Type *last, *next;
+   
+   last = NULL;
+   next = Pipe_Cmd_Table;
+   while (next != NULL)
+     {
+	if (next->fp == fp)
+	  {
+	     if (last != NULL)
+	       last->next = next->next;
+	     else 
+	       Pipe_Cmd_Table = next->next;
+	     
+	     SLang_free_slstring (next->cmd);
+	     slrn_free ((char *) next);
+	     return;
+	  }
+	
+	last = next;
+	next = next->next;
+     }
+   
+   /* should not be reached */
+}
+
+static char *get_pipe_cmd (FILE *fp)
+{
+   Pipe_Cmd_Table_Type *p;
+   
+   p = Pipe_Cmd_Table;
+   while (p != NULL)
+     {
+	if (p->fp == fp)
+	  return p->cmd;
+	
+	p = p->next;
+     }
+   
+   return "**UNKNOWN**";
+}
+#endif 				       /* SLRN_HAS_PIPING */
 int slrn_pclose (FILE *fp) /*{{{*/
 {
 #if SLRN_HAS_PIPING
@@ -859,10 +957,13 @@ int slrn_pclose (FILE *fp) /*{{{*/
    if (ret)
      {
 	char buf[SLRN_MAX_PATH_LEN];
-	fprintf (stderr, "Command returned exit status %d.  Press RETURN.\n", ret);
+	fprintf (stderr, "Command %s returned exit status %d.  Press RETURN.\n", 
+		 get_pipe_cmd (fp), ret);
 	fgets (buf, sizeof(buf), stdin);
      }
 
+   delete_pipe_cmd (fp);
+
    slrn_set_display_state (SLRN_TTY_INIT | SLRN_SMG_INIT);
    return 0;
 #else
@@ -887,6 +988,8 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
 	fgets (buf, sizeof(buf), stdin);
 	slrn_set_display_state (SLRN_TTY_INIT | SLRN_SMG_INIT);
      }
+   else (void) store_pipe_cmd (cmd, fp);
+
    return fp;
 #else
    return NULL;
@@ -926,7 +1029,18 @@ static int create_edit_command (char *edit, char *cmd, char *file, unsigned int
 	  }
 	p++;
      }
-   
+
+#if defined(IBMPC_SYSTEM)
+   /* Convert editor pathnames from / form to \\ form.  I wonder what 
+    * happens when the pathname contains a space.  Hmmm...
+    */
+   p = edit;
+   while ((*p != ' ') && (*p != '\t') && (*p != 0))
+     {
+	if (*p == '/') *p = '\\';
+	p++;
+     }
+#endif
    
    /* No %d, %s */
    
@@ -954,12 +1068,20 @@ static int create_edit_command (char *edit, char *cmd, char *file, unsigned int
 }
 
 /*}}}*/
-int slrn_edit_file (char *editor, char *file, unsigned int line) /*{{{*/
+
+/* This function returns -1 upon failure, -2 upon unmodified edit */
+int slrn_edit_file (char *editor, char *file, unsigned int line, 
+		    int check_mtime) /*{{{*/
 {
-   char buf[512];
-   char editbuf[512];
+   char buf[1024];
+   char editbuf[1024];
    int ret;
-   
+   unsigned long mtime = 0;
+   struct stat st;
+
+   if (Slrn_Abort_Unmodified == 0)
+     check_mtime = 0;
+     
    if (editor == NULL)
      editor = Slrn_Editor;
 
@@ -983,17 +1105,28 @@ int slrn_edit_file (char *editor, char *file, unsigned int line) /*{{{*/
      }
    
    strcpy (editbuf, editor);
+
    if (0 == create_edit_command(editbuf, buf, file, line)) return -1;
-   
+
+   if (check_mtime
+       && (0 == stat (file, &st)))
+     mtime = (unsigned long) st.st_mtime;
+       
    ret = slrn_posix_system (buf, 1);
-   
+
    /* Am I the only one who thinks this is a good idea?? */
    if (Slrn_TT_Initialized) while (SLang_input_pending (5))
+     SLang_flush_input ();
+
+   if (check_mtime
+       && (ret != -1)
+       && (0 == stat (file, &st))
+       && (mtime == (unsigned long) st.st_mtime))
      {
-	SLang_flush_input ();
+	slrn_message ("File was not modified");
+	ret = -2;
      }
-   
-   /* slrn_redraw (); */
+
    return ret;
 }
 
@@ -1141,7 +1274,7 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    int save_slang_error;
    
    Slrn_Full_Screen_Update = 1;
-
+   
    strncpy (prompt_buf, prompt, sizeof (prompt_buf));
    prompt_buf[sizeof(prompt_buf) - 1] = 0;
    len = strlen (prompt);
@@ -1512,14 +1645,92 @@ int slrn_is_fqdn (char *h) /*{{{*/
      return 0;
    
    p = slrn_strchr (h, '.');
-   return ((p != NULL) && (p != h));
+   if ((p == NULL) || (p == h))
+     return 0;
+   
+   /* Make sure it does not end in a '.' */
+   if (p [strlen(p)-1] == '.')
+     return 0;
+   
+   return 1;
 }
-
 /*}}}*/
-void slrn_get_user_info (void) /*{{{*/
+
+/* Try to get a fully qualified domain name. */
+static char *get_hostname (void)
 {
    struct hostent *host_entry;
-   char *name, *host;
+   char buf[MAX_HOST_NAME_LEN + 1];
+
+   host_entry = NULL;
+   if ((-1 == gethostname (buf, MAX_HOST_NAME_LEN))
+       || (*buf == 0))
+     return NULL;
+
+   /* gethostname may not provide the full name so use gethostbyname
+    * to get more information.  Why isn't there a simplified interface to
+    * get the FQDN!!!!
+    */
+   host_entry = gethostbyname (buf);
+
+#if defined(TRY_AGAIN) && !defined(MULTINET)
+   if ((host_entry == NULL) && (h_errno == TRY_AGAIN))
+     {
+	sleep (2);
+	host_entry = gethostbyname (buf);
+     }
+#endif
+
+   if ((host_entry != NULL)
+       && (host_entry->h_name != NULL)
+       && (host_entry->h_name[0] != 0))
+     {
+	char **aliases;
+
+	if ((0 == slrn_is_fqdn ((char *)host_entry->h_name))
+	    && (NULL !=	(aliases = host_entry->h_aliases)))
+	  {
+	     while (*aliases != NULL)
+	       {
+		  if (slrn_is_fqdn (*aliases))
+		    return slrn_safe_strmalloc (*aliases);
+		  aliases++;
+	       }
+	  }
+
+	return slrn_safe_strmalloc ((char *)host_entry->h_name);
+     }
+   
+   return slrn_safe_strmalloc (buf);
+}
+
+#ifdef OUR_HOSTNAME
+/* Returns a pointer to a statically allocated area */
+static char *get_host_from_filename (char *file)
+{
+   FILE *fp;
+   char *host;
+   static char line[MAX_HOST_NAME_LEN + 1];
+
+   if (NULL == (fp = fopen (file, "r")))
+     return NULL;
+   
+   host = NULL;
+   if (NULL != fgets (line, sizeof (line), fp))
+     {
+	slrn_trim_string (line);
+	if (slrn_is_fqdn (line))
+	  host = line;
+     }
+
+   fclose(fp);
+   return host;
+}
+#endif				       /* OUR_HOSTNAME */
+
+void slrn_get_user_info (void) /*{{{*/
+{
+   char *name, *host, *host1;
 #ifdef HAS_PASSWORD_CODE
    struct passwd *pw;
 #endif
@@ -1531,111 +1742,63 @@ void slrn_get_user_info (void) /*{{{*/
    Slrn_Courtesy_CC_Message = slrn_safe_strmalloc ("[This message has also been posted.]");
    
    /* Now get default values for rest. */
-   
-   host = Slrn_User_Info.host;
-   
-   /* gethostname may not provide the full name so use gethostbyname
-    * to get more information.  Why isn't there a simplified interface to
-    * get the FQDN!!!!
-    */
-   host_entry = NULL;
-   if (-1 != gethostname (host, MAX_HOST_NAME_LEN))
-     host_entry = gethostbyname (host);
-#if defined(TRY_AGAIN) && !defined(MULTINET)
-   if ((host_entry == NULL) && (h_errno == TRY_AGAIN))
+   host = get_hostname ();
+   if (host != NULL)
      {
-	sleep (2);
-	host_entry = gethostbyname (host);
-     }
-#endif
-   
-   if ((host_entry == NULL) || (host_entry->h_name == NULL))
-     *host = 0;
-   else
-     {
-	if (slrn_is_fqdn ((char *)host_entry->h_name))
-	  {
-	     Slrn_User_Info.posting_host = slrn_safe_strmalloc ((char *)host_entry->h_name);
-	  }
-	else
-	  {
-	     char **aliases;
-	     aliases = host_entry->h_aliases;
-	     if (aliases != NULL) while (*aliases != NULL)
-	       {
-		  if (slrn_is_fqdn (*aliases))
-		    {
-		       Slrn_User_Info.posting_host = slrn_safe_strmalloc (*aliases);
-		       break;
-		    }
-		  aliases++;
-	       }
-	  }
-	
-	if (Slrn_User_Info.posting_host == NULL)
-	  {
-	     strcpy (host, (char *) host_entry->h_name);
-	  }
-	else strcpy (host, Slrn_User_Info.posting_host);
-     }
-   
+	if (slrn_is_fqdn (host))
+	  Slrn_User_Info.posting_host = slrn_safe_strmalloc (host);
 #if defined(USE_DOMAIN_NAME) && defined(MY_DOMAIN_NAME)
-   if (*host && (0 == slrn_is_fqdn (host)))
-     {
-	char *my_domain_name;
-	unsigned int len;
-	
-	len = strlen (host);
-	my_domain_name = MY_DOMAIN_NAME;
-	if (*my_domain_name != '.')
+	else if (MY_DOMAIN_NAME != NULL)
 	  {
-	     host[len++] = '.';
+	     char *my_domain_name;
+	     unsigned int len;
+
+	     my_domain_name = MY_DOMAIN_NAME;
+	     if (*my_domain_name == '.')
+	       my_domain_name++;
+
+	     len = strlen (host);
+	     host1 = slrn_safe_malloc (len + strlen (my_domain_name) + 2);
+	     strcpy (host1, host);
+	     host1[len] = '.';
+	     strcpy (host1 + len + 1, my_domain_name);
+	     slrn_free (host);
+	     host = host1;
 	  }
-	strcpy (host + len, my_domain_name);
+#endif				       /* USE_DOMAIN_NAME */
      }
-#endif
+
+   if (Slrn_User_Info.posting_host == NULL)
+     slrn_message_now ("***Warning: unable to find a fully-qualified host name.");
+
+   /* Allow user chance to specify another hostname.  However, it will not
+    * affect the posting host.
+    */
+   if ((NULL != (host1 = getenv ("HOSTNAME")))
+       && (0 == slrn_is_fqdn (host1)))
+     host1 = NULL;
+
    /* If a value was compiled in, use it. */
 #ifdef OUR_HOSTNAME
-# ifdef __unix__
-   /* If the value is a file, read the file for the FQDN */
-   if (slrn_is_absolute_path (OUR_HOSTNAME))
+   if (host1 == NULL)
      {
-	FILE *fp;
-	
-	if (NULL != (fp = fopen(OUR_HOSTNAME, "r")))
-	  {
-	   char tmphost[MAX_HOST_NAME_LEN];
-	     
-	     if (fgets(tmphost, MAX_HOST_NAME_LEN , fp) != NULL)
-	       {
-		  char *p;
-		  
-		  if ((p = slrn_strchr (tmphost, '\n')) != NULL)
-		    *p = '\0';
-	       /* If the entry is valid, we're done */
-		  if (slrn_is_fqdn(tmphost))
-		    strcpy(host, tmphost);
-	       }
-	     fclose(fp);
-	  }
+	host1 = OUR_HOSTNAME;
+	/* If the value is a file, read the file for the FQDN */
+	if (slrn_is_absolute_path (host1))
+	  host1 = get_host_from_filename (OUR_HOSTNAME);
      }
-   else
-# endif
-     if (slrn_is_fqdn (OUR_HOSTNAME)
-	 && strlen(OUR_HOSTNAME) < MAX_HOST_NAME_LEN)
-       strcpy (host, OUR_HOSTNAME);
 #endif
-
-   /* Allow user chance to over ride this.  Until now, host was a pointer
-    * to Slrn_User_Info.host.
-    */
-   if ((NULL != (host = getenv ("HOSTNAME")))
-       && slrn_is_fqdn (host))
-     {
-	strncpy (Slrn_User_Info.host, host, MAX_HOST_NAME_LEN);
-	Slrn_User_Info.host[MAX_HOST_NAME_LEN - 1] = 0;
-     }
    
+   if ((host1 != NULL)
+       && slrn_is_fqdn (host1))
+     {
+	slrn_free (host);
+	host = slrn_safe_strmalloc (host1);
+     }
+
+   /* Finally!! */
+   Slrn_User_Info.hostname = host;
+
 #ifdef VMS
    name = slrn_vms_getlogin();
 #else
@@ -1721,3 +1884,62 @@ void slrn_get_user_info (void) /*{{{*/
 }
 
 /*}}}*/
+
+static char *_slrn_strcat (char *a, char *b, unsigned int n)
+{
+   unsigned int max;
+   unsigned int len = strlen (a);
+   
+   n--;
+   
+   if (n > len)
+     {
+	max = n - len;
+	strncpy (a + len, b, max);
+	a[n] = 0;
+     }
+   return a;
+}
+
+char *slrn_make_from_string (void)
+{
+   char *username, *hostname, *realname;
+   static char buf[256];
+
+   *buf = 0;
+
+#if SLRN_HAS_SLANG
+   if ((1 == SLang_run_hooks ("make_from_string_hook", 0))
+       && (0 == SLang_pop_slstring (&username)))
+     {
+	if (*username != 0)
+	  {
+	     _slrn_strcat (buf, username, sizeof (buf));
+	     SLang_free_slstring (username);
+	     return buf;
+	  }
+	SLang_free_slstring (username);
+	/* Drop through to default */
+     }
+#endif
+
+   if (NULL == (username = Slrn_User_Info.username))
+     username = "";
+
+   if (NULL == (hostname = Slrn_User_Info.hostname))
+     hostname = "";
+
+   if (NULL == (realname = Slrn_User_Info.realname))
+     realname = "";
+
+   /* Build string of the form "username@hostname (realname)" */
+   _slrn_strcat (buf, username, sizeof(buf));
+   _slrn_strcat (buf, "@", sizeof(buf));
+   _slrn_strcat (buf, hostname, sizeof(buf));
+   _slrn_strcat (buf, " (", sizeof(buf));
+   _slrn_strcat (buf, realname, sizeof(buf));
+   _slrn_strcat (buf, ")", sizeof(buf));
+   
+   return buf;
+}
+
