@@ -403,7 +403,7 @@ static int _nntp_connect_server (NNTP_Type *s)
 
    slrn_message_now ("Connecting to host %s ...", s->host);
 
-   if (NULL == (s->tcp = sltcp_open_connection (s->host, s->port)))
+   if (NULL == (s->tcp = sltcp_open_connection (s->host, s->port, s->use_ssh)))
      return -1;
 
    s->init_state = 1;
@@ -515,22 +515,40 @@ char *nntp_get_server_name (void)
 static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
 {
    char *a, *b, *bmax;
+   int def_port = 119;
+   char *def_service = "nntp";
 
    a = host;
+   if (0 == strncmp (a, "snews://", 8))
+     {
+	s->use_ssh = 1;
+	a += 8;
+     }
+   else if (0 == strncmp (a, "news://", 7))
+     {
+	s->use_ssh = 0;
+	a += 7;
+     }
+
    b = s->host;
    bmax = b + NNTP_MAX_HOST_LEN;
    while ((*a != 0) && (*a != ':') && (b < bmax))
      *b++ = *a++;
    *b = 0;
 
+   if (s->use_ssh)
+     {
+	def_port = 563;
+	def_service = "nntps";
+     }
+
    if (port <= 0)
      {
 	if (*a == ':')
 	  port = atoi (a + 1);
-	else if (-1 == (port = sltcp_map_service_to_port ("nntp")))
-	  port = 119;
+	else if (-1 == (port = sltcp_map_service_to_port (def_service)))
+	  port = def_port;
      }
-
    s->port = port;
 
    return 0;
@@ -611,7 +629,7 @@ int nntp_reconnect_server (NNTP_Type *s)
 
 int nntp_check_connection (NNTP_Type *s)
 {
-   if ((s == NULL) || (s->tcp == NULL) || (s->tcp->tcp_fd == -1))
+   if ((s == NULL) || (-1 == sltcp_get_fd (s->tcp)))
      return -1;
 
    return 0;
