@@ -1202,6 +1202,8 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
    
    if (use_active) Slrn_List_Active_File = 1;
    if (use_mouse) Slrn_Use_Mouse = -1; /* -1 forces it. */
+   if (use_color == 1) SLtt_Use_Ansi_Colors = 1;
+   else if (use_color == -1) SLtt_Use_Ansi_Colors = 0;
 
    if (dsc_flag)
      {
