@@ -951,7 +951,7 @@ static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
 	elapsed_time = 1;
      }
    
-   in = s->tcp->bytes_in;
+   in = sltcp_get_num_input_bytes (s->tcp);
 
    hour = elapsed_time / 3600;
    min = (elapsed_time - 3600 * hour) / 60;
@@ -1655,7 +1655,7 @@ static int open_servers (char *host) /*{{{*/
 
    NNTP_Connection_Lost_Hook = connection_lost_hook;
 
-   Pull_Server->tcp->bytes_in = Pull_Server->tcp->bytes_out = 0;
+   sltcp_reset_num_io_bytes (Pull_Server->tcp);
    time (&Start_Time);
    
    /* Probe for XHDR now because DNEWS does not handle probing later properly. */
@@ -1695,8 +1695,8 @@ static void close_servers (void) /*{{{*/
      {
 	if (Pull_Server->tcp != NULL) 
 	  {
-	     bytes_in = Pull_Server->tcp->bytes_in;
-	     bytes_out = Pull_Server->tcp->bytes_out;
+	     bytes_in = sltcp_get_num_input_bytes (Pull_Server->tcp);
+	     bytes_out = sltcp_get_num_output_bytes (Pull_Server->tcp);
 	  }
 	nntp_close_server (Pull_Server);
      }
