@@ -19,6 +19,7 @@
  Mass Ave, Cambridge, MA 02139, USA. 
 */
 #include "config.h"
+#include "slrnfeat.h"
 
 /*{{{ Include Files */
 
@@ -102,8 +103,34 @@ extern int h_errno;
 
 #include <slang.h>
 
+#if SLTCP_HAS_SSL_SUPPORT
+# include <openssl/ssl.h>
+# include <openssl/err.h>
+# include <openssl/rand.h>
+#endif
+
 #include "sltcp.h"
 
+struct _SLTCP_Type
+{
+   int tcp_fd;
+#define SLTCP_EOF_FLAG	0x1
+   unsigned int tcp_flags;
+   unsigned char *tcp_write_ptr;
+   unsigned char *tcp_write_ptr_min;
+   unsigned char *tcp_read_ptr_max;
+   unsigned char *tcp_read_ptr;
+#define SLTCP_BUF_SIZE 8192
+   unsigned char tcp_read_buf [SLTCP_BUF_SIZE];
+   unsigned char tcp_write_buf [SLTCP_BUF_SIZE];
+   unsigned long bytes_out;
+   unsigned long bytes_in;
+#if SLTCP_HAS_SSL_SUPPORT
+   SSL *ssl;
+#endif
+};
+
+
 /*}}}*/
 
 #if defined(__BEOS__) || defined(USE_WINSOCK_SLTCP)
@@ -321,16 +348,149 @@ static int get_tcp_socket (char *host, int port) /*{{{*/
 
 /*}}}*/
 #endif	   			       /* HAVE_SIGLONGJMP */
-	   
-SLTCP_Type *sltcp_open_connection (char *host, int port) /*{{{*/
+
+#if SLTCP_HAS_SSL_SUPPORT
+static SSL_CTX *This_SSL_Ctx;
+static void deinit_ssl (void)
+{
+   if (This_SSL_Ctx == NULL)
+     return;
+   SSL_CTX_free (This_SSL_Ctx);
+   This_SSL_Ctx = NULL;
+}
+
+static void dump_ssl_error_0 (void)
+{
+   int err;
+   
+   while (0 != (err = ERR_get_error()))
+     fprintf (stderr, "%s\n", ERR_error_string(err, 0));
+}
+
+unsigned long Fast_Random;
+static unsigned long fast_random (void)
+{
+   return (Fast_Random = Fast_Random * 69069U + 1013904243U);
+}
+
+static int init_ssl_random (void)
+{
+   time_t t;
+   pid_t pid;
+   unsigned int count;
+
+   if (RAND_status ())
+     return 0;
+   
+   time (&t);
+   pid = getpid ();
+   RAND_seed ((unsigned char *) &pid, sizeof (time_t));
+   RAND_seed ((unsigned char *) &t, sizeof (pid_t));
+   
+   RAND_bytes ((unsigned char *) &Fast_Random, sizeof(long));
+   
+   count = 0;
+   while ((count < 10000) 
+	  && (0 == RAND_status ()))
+     {
+	unsigned long r = fast_random ();
+	RAND_seed (&r, sizeof (unsigned long));
+     }
+   if (RAND_status ())
+     return 0;
+   
+   fprintf (stderr, "Unable to generate enough entropy\n");
+   return -1;
+}
+
+
+static SSL *alloc_ssl (void)
+{
+   SSL *ssl;
+
+   if (This_SSL_Ctx == NULL)
+     {
+	SSL_CTX *c;
+
+	SSLeay_add_ssl_algorithms ();
+	c = SSL_CTX_new(SSLv23_client_method());
+	SSL_load_error_strings ();
+	if (c == NULL)
+	  {
+	     dump_ssl_error_0 ();
+	     fprintf (stderr, "SSL_CTX_new failed.\n");
+	     return NULL;
+	  }
+	This_SSL_Ctx = c;
+	atexit (deinit_ssl);
+	
+	if (-1 == init_ssl_random ())
+	  return NULL;
+     }
+   
+   ssl = SSL_new (This_SSL_Ctx);
+   if (ssl != NULL)
+     return ssl;
+   
+   dump_ssl_error_0 ();
+   fprintf (stderr, "SSL_new failed\n");
+   return NULL;
+}
+
+static void dealloc_ssl (SSL *ssl)
+{
+   if (ssl != NULL)
+     SSL_free (ssl);
+}
+
+static void dump_ssl_error (SSL *ssl, char *msg, int status)
+{
+   switch (SSL_get_error (ssl, status))
+     {
+      case SSL_ERROR_NONE:
+	break;
+      case SSL_ERROR_ZERO_RETURN:
+	fprintf (stderr, "Unexpected error: SSL connection closed\n");
+	break;
+      case SSL_ERROR_SYSCALL:
+	fprintf (stderr, "System call failed: errno = %d\n", errno);
+	break;
+      case SSL_ERROR_SSL:
+	fprintf (stderr, "Possible protocol error\n");
+	break;
+     }
+   dump_ssl_error_0 ();
+   if (msg != NULL) fprintf (stderr, "%s failed\n", msg);
+}
+#endif				       /* SLTCP_HAS_SSL_SUPPORT */
+
+SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
 {
    int fd;
    SLTCP_Type *tcp;
-   
+#if SLTCP_HAS_SSL_SUPPORT
+   SSL *ssl = NULL;
+   int status;
+#endif
+
+   if (use_ssl)
+     {
+#if SLTCP_HAS_SSL_SUPPORT
+	if (NULL == (ssl = alloc_ssl ()))
+	  return NULL;
+#else
+	fprintf (stderr, "\n\n*** This program does not support SSL\n");
+	return NULL;
+#endif
+     }
+
    tcp = (SLTCP_Type *) SLMALLOC (sizeof (SLTCP_Type));
    if (tcp == NULL)
      {
 	fprintf (stderr, "Memory Allocation Failure.\n");
+#if SLTCP_HAS_SSL_SUPPORT
+	dealloc_ssl (ssl);
+#endif
 	return NULL;
      }
    memset ((char *) tcp, 0, sizeof (SLTCP_Type));
@@ -343,22 +503,124 @@ SLTCP_Type *sltcp_open_connection (char *host, int port) /*{{{*/
    if (fd == -1) 
      {
 	SLFREE (tcp);
+#if SLTCP_HAS_SSL_SUPPORT
+	dealloc_ssl (ssl);
+#endif
 	return NULL;
      }
 
    tcp->tcp_fd = fd;
+
+#if SLTCP_HAS_SSL_SUPPORT
+   if (ssl == NULL)
+     return tcp;
+   tcp->ssl = ssl;
+
+   if (1 != SSL_set_fd (ssl, fd))
+     {
+	/* Yuk.  SSL_set_fd returns 0 upon failure */
+	fprintf (stderr, "SSL_set_fd failed\n");
+	sltcp_close (tcp);
+	return NULL;
+     }
    
+   status = SSL_connect (ssl);
+   if (status == 1)
+     return tcp;
+
+   dump_ssl_error (ssl, "SSL_connect", status);
+   sltcp_close (tcp);
+   return NULL;
+#else
    return tcp;
+#endif
 }
 
 /*}}}*/
 
+static int check_errno (void)
+{
+#ifdef EAGAIN
+   if (errno == EAGAIN) 
+     {
+	sleep (1);
+	return 0;
+     }
+#endif
+#ifdef EWOULDBLOCK
+   if (errno == EWOULDBLOCK)
+     {
+	sleep (1);
+	return 0;
+     }
+#endif
+#ifdef EINTR
+   if (errno == EINTR) 
+     {
+	if (-1 == sys_call_interrupted_hook ())
+	  return -1;
+	return 0;
+     }
+#endif
+   return -1;
+}
+
+#if SLTCP_HAS_SSL_SUPPORT
+static unsigned int do_ssl_write (SLTCP_Type *tcp, unsigned char *buf, unsigned int len)
+{
+   unsigned int total;
+   SSL *ssl = tcp->ssl;
+
+   total = 0;
+   ssl = tcp->ssl;
+
+   while (total != len)
+     {
+	int nwrite;
+	
+	errno = 0;
+	nwrite = SSL_write (ssl, (char *) buf, (len - total));
+
+	if (nwrite > 0)
+	  {
+	     total += (unsigned int) nwrite;
+	     tcp->bytes_out += total;
+	     continue;
+	  }
+	
+	switch (SSL_get_error (ssl, nwrite))
+	  {
+	   default:
+	   case SSL_ERROR_ZERO_RETURN: /* Connection closed */
+	     return total;
+
+	   case SSL_ERROR_WANT_READ:
+	   case SSL_ERROR_WANT_WRITE:
+	     if (-1 == sys_call_interrupted_hook ())
+	       return total;
+	     break;		       /* try again */
+	     
+	   case SSL_ERROR_SYSCALL:
+	     if (-1 == check_errno ())
+	       return total;
+	     break;
+	  }
+     }
+
+   return total;
+}
+#endif
+
 static unsigned int do_write (SLTCP_Type *tcp, unsigned char *buf, unsigned int len) /*{{{*/
 {
    unsigned int total;
    int nwrite;
    int fd;
-   
+
+#if SLTCP_HAS_SSL_SUPPORT
+   if (tcp->ssl != NULL)
+     return do_ssl_write (tcp, buf, len);
+#endif
    total = 0;
    fd = tcp->tcp_fd;
    
@@ -368,33 +630,11 @@ static unsigned int do_write (SLTCP_Type *tcp, unsigned char *buf, unsigned int
 	
 	if (nwrite == -1)
 	  {
-#ifdef EAGAIN
-	     if (errno == EAGAIN) 
-	       {
-		  sleep (1);
-		  continue;
-	       }
-#endif
-#ifdef EWOULDBLOCK
-	     if (errno == EWOULDBLOCK)
-	       {
-		  sleep (1);
-		  continue;
-	       }
-#endif
-#ifdef EINTR
-	     if (errno == EINTR) 
-	       {
-		  if (-1 == sys_call_interrupted_hook ())
-		    {
-		       return total;
-		    }
-		  continue;
-	       }
-#endif
-	     break;
+	     if (-1 == check_errno ())
+	       break;
+	     continue;
 	  }
-	
+
 	total += (unsigned int) nwrite;
      }
    
@@ -441,7 +681,10 @@ int sltcp_close_socket (SLTCP_Type *tcp)
    if ((tcp == NULL)
        || (tcp->tcp_fd == -1))
      return 0;
-   
+#if SLTCP_HAS_SSL_SUPPORT
+   dealloc_ssl (tcp->ssl);
+   tcp->ssl = NULL;
+#endif
    status = 0;
    while (-1 == SLTCP_CLOSE (tcp->tcp_fd))
      {
@@ -521,6 +764,35 @@ static int wait_for_input (int fd) /*{{{*/
 /*}}}*/
 #endif
 
+#if SLTCP_HAS_SSL_SUPPORT
+static int do_ssl_read (SSL *ssl, char *buf, unsigned int len)
+{
+   int nread;
+   while ((nread = SSL_read (ssl, buf, len)) <= 0)
+     {
+	switch (SSL_get_error (ssl, nread))
+	  {
+	   default:
+	   case SSL_ERROR_ZERO_RETURN: /* Connection closed */
+	     return -1;
+
+	   case SSL_ERROR_WANT_READ:
+	   case SSL_ERROR_WANT_WRITE:
+	     if (-1 == sys_call_interrupted_hook ())
+	       return -1;
+	     break;
+	     
+	   case SSL_ERROR_SYSCALL:
+	     if (-1 == check_errno ())
+	       return -1;
+	     break;
+	  }
+	
+     }
+   
+   return nread;
+}
+#endif
 static int do_read (SLTCP_Type *tcp) /*{{{*/
 {   
    int nread, fd;
@@ -541,21 +813,20 @@ static int do_read (SLTCP_Type *tcp) /*{{{*/
      return -1;
 #endif
 
-   while (-1 == (nread = SLTCP_READ (fd, (char *)tcp->tcp_read_ptr, SLTCP_BUF_SIZE)))
-     {
-	if (errno == EINTR)
-	  {
-	     if (-1 == sys_call_interrupted_hook ())
-	       return -1;
-	     continue;
-	  }
-	return -1;
-     }
-   
+#if SLTCP_HAS_SSL_SUPPORT
+   if (tcp->ssl != NULL)
+     nread = do_ssl_read (tcp->ssl, (char *)tcp->tcp_read_ptr, SLTCP_BUF_SIZE);
+   else 
+#endif
+     while (-1 == (nread = SLTCP_READ (fd, (char *)tcp->tcp_read_ptr, SLTCP_BUF_SIZE)))
+       {
+	  if (-1 == check_errno ())
+	    return -1;
+       }
+
    if (nread == 0)
      tcp->tcp_flags |= SLTCP_EOF_FLAG;
-	
-	
+
    tcp->tcp_read_ptr_max += (unsigned int) nread;
    tcp->bytes_in += (unsigned int) nread;
    
@@ -766,6 +1037,37 @@ int sltcp_close_sltcp (void)
    return 0;
 }
 
+unsigned int sltcp_get_num_input_bytes (SLTCP_Type *tcp)
+{
+   if (tcp == NULL)
+     return 0;
+   return tcp->bytes_in;
+}
+
+unsigned int sltcp_get_num_output_bytes (SLTCP_Type *tcp)
+{
+   if (tcp == NULL)
+     return 0;
+   return tcp->bytes_out;
+}
+
+int sltcp_reset_num_io_bytes (SLTCP_Type *tcp)
+{
+   if (tcp == NULL)
+     return -1;
+   tcp->bytes_in = tcp->bytes_out = 0;
+   return 0;
+}
+
+int sltcp_get_fd (SLTCP_Type *tcp)
+{
+   if (tcp == NULL)
+     return -1;
+   return tcp->tcp_fd;
+}
+
+
+
 #if 0 && defined(USE_WINSOCK_SLTCP)
 /* Some winsock notes */
 
