@@ -23,25 +23,10 @@
 
 #include <stdarg.h>
 
-typedef struct 
-{
-   int tcp_fd;
-#define SLTCP_EOF_FLAG	0x1
-   unsigned int tcp_flags;
-   unsigned char *tcp_write_ptr;
-   unsigned char *tcp_write_ptr_min;
-   unsigned char *tcp_read_ptr_max;
-   unsigned char *tcp_read_ptr;
-#define SLTCP_BUF_SIZE 8192
-   unsigned char tcp_read_buf [SLTCP_BUF_SIZE];
-   unsigned char tcp_write_buf [SLTCP_BUF_SIZE];
-   unsigned long bytes_out;
-   unsigned long bytes_in;
-}
-SLTCP_Type;
+typedef struct _SLTCP_Type SLTCP_Type;
 
 extern int sltcp_close_socket (SLTCP_Type *);
-extern SLTCP_Type *sltcp_open_connection (char *, int);
+extern SLTCP_Type *sltcp_open_connection (char *, int, int);
 extern int sltcp_close (SLTCP_Type *);
 extern unsigned int sltcp_read (SLTCP_Type *, char *, unsigned int);
 extern unsigned int sltcp_write (SLTCP_Type *, char *, unsigned int);
@@ -56,6 +41,10 @@ extern int sltcp_fgets (SLTCP_Type *, char *, unsigned int);
 extern int sltcp_open_sltcp (void);
 extern int sltcp_close_sltcp (void);
 
+extern unsigned int sltcp_get_num_input_bytes (SLTCP_Type *);
+extern unsigned int sltcp_get_num_output_bytes (SLTCP_Type *);
+extern int sltcp_reset_num_io_bytes (SLTCP_Type *);
+extern int sltcp_get_fd (SLTCP_Type *);
 #endif
 
 #endif
